package week4Exercises;


import com.jogamp.opengl.GL;
import com.jogamp.opengl.GL2;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.GLCapabilities;
import com.jogamp.opengl.GLEventListener;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.awt.GLJPanel;
import com.jogamp.opengl.glu.GLU;
import javax.swing.JFrame;



// At the end of week 4 lectures I gave some exercises.
// They are at the end of the week4 slides.
// These are the solutions

public class Exercise implements GLEventListener {
	
  
    public static void main(String[] args) {
        // initialisation
        GLProfile glp = GLProfile.getDefault();
        GLCapabilities caps = new GLCapabilities(glp);
        
        // create a panel to draw on
        GLJPanel panel = new GLJPanel(caps);

        // put it in a JFrame
        final JFrame jframe = new JFrame("Exercise");
        jframe.setSize(300, 300);
        jframe.add(panel);
        jframe.setVisible(true);
       
        jframe.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        panel.addGLEventListener(new Exercise());
              
    }

    double [] cross(double u [], double v[]){
    	double crossProduct[] = new double[3];
    	crossProduct[0] = u[1]*v[2] - u[2]*v[1];
    	crossProduct[1] = u[2]*v[0] - u[0]*v[2];
    	crossProduct[2] = u[0]*v[1] - u[1]*v[0];
    	
    	return crossProduct;
    }
    
    //Assumes correct winding order
    double [] getNormal(double[] p0, double[] p1, double[] p2){
    	double u[] = {p1[0] - p0[0], p1[1] - p0[1], p1[2] - p0[2]};
    	double v[] = {p2[0] - p0[0], p2[1] - p0[1], p2[2] - p0[2]}; 	
    	return cross(u,v);   	
    }
    
    
    @Override
    public void display(GLAutoDrawable drawable) {
    	GL2 gl = drawable.getGL().getGL2();    	
    	gl.glMatrixMode(GL2.GL_MODELVIEW);
    	gl.glLoadIdentity();

    	//Move camera back
    	gl.glTranslated(0, 0, -2.5);
    	
    	gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL2.GL_DEPTH_BUFFER_BIT);
    	
    	//With winding order correct
    	double v0[] = {0,-1,-3};
    	double v1[] = {2,1,-4};
    	double v2[] = {-2,1,-4};
    	
    	double v3[] = {-0.5,0,0};
    	double v4[] = {0.5,-0.5,0};
    	double v5[] = {0.5,0.5,0};
    	
    	double faceNormal[] = getNormal(v0,v1,v2);
    	double faceNormal2[] = getNormal(v3,v4,v5);   	
   
    	gl.glBegin(GL.GL_TRIANGLES);
    	    gl.glNormal3d(faceNormal[0], faceNormal[1], faceNormal[2]);
    		gl.glVertex3dv(v0,0);
    		gl.glVertex3dv(v1,0);
    		gl.glVertex3dv(v2,0);
    	gl.glEnd();
    	    	
    	gl.glBegin(GL.GL_TRIANGLES);
    	    gl.glNormal3d(faceNormal2[0], faceNormal2[1], faceNormal2[2]);
    		gl.glVertex3dv(v3,0);
    		gl.glVertex3dv(v4,0);
    		gl.glVertex3dv(v5,0);
    	gl.glEnd();
    }

    @Override
    public void dispose(GLAutoDrawable drawable) {
    }

    @Override
    public void init(GLAutoDrawable drawable) {
    	GL2 gl = drawable.getGL().getGL2();    	
    	gl.glEnable(GL2.GL_DEPTH_TEST);
    	
    	//By enabling lighting, color is worked out differently.
    	gl.glEnable(GL2.GL_LIGHTING);
    	
    	//When you enable lighting you must still actually
    	//turn on a light such as this default light.
    	gl.glEnable(GL2.GL_LIGHT0); 
    	gl.glEnable(GL2.GL_NORMALIZE);
    	
    	//To check if our winding order is correct
    	gl.glEnable(GL2.GL_CULL_FACE);
        gl.glCullFace(GL2.GL_BACK);
  	
    }

    @Override
    public void reshape(GLAutoDrawable drawable, int x, int y, int w, int h) {
    	 GL2 gl = drawable.getGL().getGL2();
         gl.glMatrixMode(GL2.GL_PROJECTION);
         gl.glLoadIdentity();  
         
         //You can use an orthographic camera
         //gl.glOrtho(-2, 2, -2, 2, 1, 20);
         GLU glu = new GLU();
         //glu.gluPerspective(60,1,2,8);
         
         //To find equivalent settings using gl.glFrustum
         // y = near * tan (30);
         // x = aspect * y
         gl.glFrustum(-1.15, 1.15, -1.15, 1.15, 2, 8);
         
         //Can use printMatrix to check they are indeed equivalent
         
    }  
    
    static void printMatrix(GL2 gl){
      	 double[] curmat = new double[16];

      	   // Get the current matrix on the MODELVIEW stack
      	 gl.glGetDoublev(GL2.GL_PROJECTION_MATRIX, curmat, 0);

      	   // Note: If you want to multiply the matrix 'curmat' onto the
      	   // matrix stack at a later time, just use this:
      	   //      gl.glMultMatrixd(curmat, 0);

      	   // Print out the contents of this matrix in OpenGL format
      	   System.out.println("as an OpenGL matrix");
      	   for (int row = 0; row < 4; row++)
      	       for (int col = 0; col < 4; col++)
      		   // OpenGL uses column-major order for storage
      		   System.out.format("%7.7f%c", curmat[row+col*4], col==3 ? '\n':' ');

      }
    
    
}
